! @@ ts2perigr1D.prg
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!
! TS2PERIGR1D.PRG
! A.KAUFER - LSW Heidelberg
! AK            940221  first version
!               940810  CLEAN-like algorithm introduced
! .PURPOSE  
!
! execute the command :
! TS2PERIGR1D intable
! TS2PERIGR1D intable [method] [prewhiten]
!  where
!      intable   :  table with 1D time series :time :value
!      method           : test statistics for periodogram
!                           AOV
!			    SCARGLE 
!      prewhiten :  prewhiten data with 1/obs.time [Y/N]
!
DEFINE/PAR P1 ? C   "Enter table : "
DEFINE/PAR P2 SCA C "Test statistics [AOV/SCARGLE] : "
DEFINE/PAR P3 Y   C "prewhiten data with 1/obs.time [Y/N] : "
DEFINE/PAR P4 :jd24 C "Name of independent column (time): "
DEFINE/PAR P5 :vrad C "Name of dependent column: "
!
! 
define/local cols/i/1/1 0
define/local iii/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0
define/local iter/i/1/1 0
define/local niter/i/1/1 1
define/local exist/i/1/1 0
define/local pos/i/1/1 0
define/local vel/d/1/1 0.
define/local vstart/d/1/1 0.
define/local vstep/d/1/1 0.
define/local vnpix/i/1/1 0
define/local tsstart/d/1/1 0.
define/local tsend/d/1/1 0.
define/local tsstep/d/1/1 0.
define/local tsnsteps/i/1/1 0
define/local ndata/i/1/1 0
define/local los/r/1/1 0.
define/local method/c/1/3 " " all
define/local ya/r/1/2 0. all
define/local prefreq/r/1/1 0.
define/local maxpow/d/1/1 0.
!
define/local alpha/r/1/1 0.005 ! probability for level of significance, 3 sigma
!define/local alpha/r/1/1 0.05 ! probability for level of significance, 1 sigma
define/local fititer/i/1/1 15  ! number of iterations for sinefit

define/local extlen1/i/1/1 0
define/local extlen2/i/1/1 0

extlen1 = m$index(mid$types(1:8)," ")-1
extlen2 = m$index(mid$types(9:16)," ")+8

!
write/key method/c/1/3 {p2}
write/out "---------------------------------------------"
write/out " method {method} is used for test statistics " 
write/out "---------------------------------------------"
!
!set/context tsa
! order MUST be equal 1 for CLEAN !
ordertsa = 1
!
set/gra pmode=1
!
copy/tab {p1} &tab
crea/colu &tab :time " " G8.3 R*8
crea/colu &tab :value " " G8.3 R*8
comp/tabl &tab :time = {p4}
comp/tabl &tab :value = {p5}
plo/tab   &tab :time :value
!
normalize/tsa &tab ? :time
normalize/tsa &tab ? :value
!
copy/tab &tab {p1}syn
!
!band/tsa &tab
!
tsstep   = steptsa
tsstart  = starttsa
tsnsteps = nsteps
tsend    = tsstart+tsnsteps*tsstep
!
write/out
write/out "The following settings are used: "
show/tsa
!
copy/dk &tab.{mid$types(9:{extlen2})} tblcontr/i/4/4 ndata
write/out "-----------------------------------------"
write/out " {ndata} data points found"
write/out "-----------------------------------------"
!
@@ TSPROC:fdistpp 2 {ndata} {alpha}
los = outputr(1)
write/out "-----------------------------------------"
!
! prewhiten data 
!
if "{p3}" .eq. "Y" then
 stat/tab &tab :time
 prefreq = 1./(outputr(2)-outputr(1))
 sinefit/tsa &tab {p1} {prefreq} 1 1
 copy/tab {p1} &tab
 write/descr &tab.{mid$types(9:{extlen2})} Prefreq/r/1/1 {prefreq}
 write/out "prewhitened with f={prefreq}"
 plo/tab &tab :time :value 
endif
!
! create table for fitted frequencies, phases
!
 crea/tab &tabf 20 1000
 crea/col &tabf :step " " I4 I*2
 crea/col &tabf :freq " " D12.4 R*8
 crea/col &tabf :dfreq " " D12.4 R*8
 crea/col &tabf :phmt " " D12.4 R*8
 crea/col &tabf :dphmt " " D12.4 R*8
!columns for fit coefficients
 j = 2*ordertsa+1
 do i = 1 {j}
  crea/col &tabf :c{i} " " D12.4 R*8   
 enddo
 crea/col &tabf :maxpow " " D12.4 R*8
!
! fix y-axis range
!
 power/tsa &tab &pow ? ? ? 
 stat/ima &pow [<,@1:>,@1]
 ya(1) = outputr(1)-1*outputr(4)
 ya(2) = outputr(2)+1*outputr(4)
!
next:
!
! Transform by AOV/SCARGLE
!
! compute test statistics 
!
 {p2}/tsa &tab &perigr ? ? ? ?
 plot &perigr
 over/line 1 {tsstart},{los} {tsend},{los}
!
! compute power spectrum
!
 power/tsa &tab &pow ? ? ? 
 set/gra ya={ya(1)},{ya(2)}
 plot &pow
 set/gra ya
!
!
! CLEAN: find freq with max power, fit sinusoid -> freq, phase at mean time
!
! clean only if significant peaks exist 
!
 stat/ima &perigr
 if {outputr(2)} .gt. {los} then
!
  @@ TSPROC:tsawdt &pow
  maxpow = outputd(11)
  write/out maxpow = {maxpow}
!
  @@ TSPROC:tsasin &tab {p1}CLEAN{niter} ? ? {fititer}
!
! write results (frequency, phase at mean time) to table
  copy/kd ordertsa/i/1/1  &tabf.{mid$types(9:{extlen2})} order/i/1/1
  &tabf,:step,@{niter} =  {niter}
  i = 2*ordertsa+1
  i = i+1
  &tabf,:freq,@{niter} =  {outputd({i})}
  i = i+1
  &tabf,:dfreq,@{niter} =  {outputd({i})}
  i = i+1
  &tabf,:phmt,@{niter} =  {outputd({i})}
  i = i+1
  &tabf,:dphmt,@{niter} =  {outputd({i})}
! write fit coefficients
  j = 2*ordertsa+1
  do i = 1 {j}
   &tabf,:c{i},@{niter} =  {outputd({i})}
  enddo
  &tabf,:maxpow,@{niter} =  {maxpow}
!
steptsa = tsstep 
starttsa = tsstart  
nsteps = tsnsteps
!
! inspect result of cleaning 
!
  copy/tab {p1}CLEAN{niter} &xx
  power/tsa &xx &yy ? ? ? 
  set/gra color=4
  over &yy
  set/gra color=1
!
  extract/ima {p1}PERIGR{iter} = &perigr[<,@1:>,@1]
  extract/ima {p1}POW{iter}  = &pow[<,@1:>,@1]
  extract/ima {p1}PERIGRQ{iter} = &perigr[<,@2:>,@2]
  extract/ima {p1}WIN{iter}  = &pow[<,@2:>,@2]
!
else
!
 extract/ima {p1}PERIGR{iter} = &perigr[<,@1:>,@1]
 extract/ima {p1}POW{iter}  = &pow[<,@1:>,@1]
 extract/ima {p1}PERIGRQ{iter} = &perigr[<,@2:>,@2]
 extract/ima {p1}WIN{iter}  = &pow[<,@2:>,@2]
!
 write/out "-----------------------------------------"
 write/out " No significant peaks left. STOP" 
 write/out "-----------------------------------------"
 goto final
endif
!
write/out 
write/out "-----------------------------------------"
write/out " Iteration #{iter} finished, "
inquire/key again/c/1/1 " another step [y/n] ? "
if "{again}" .eq. "y" then 
 iter = iter+1
 niter = iter+1
 copy/tab {p1}CLEAN{iter} &tab
 goto next
endif
!
final:
!sort/tab &tabf :freq
comp/tab &tabf :P = 1/:freq
comp/tab &tabf :dP = :P*:dfreq/:freq
!comp/tab &tabf :t0 = :phmt/:freq/6.2832
!comp/tab &tabf :dt0 = :dphmt/:freq/6.2832
copy/tab &tabf {p1}FIT
read/tab {p1}FIT
!
!dele/temp
end:



