! @@ ts2perigram.prg
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!
! TS2PERIGRAM.PRG
! A.KAUFER - LSW Heidelberg
! AK            940221  first version
!               940803  CLEAN-like algorithm introduced
!               941027  CLEAN algorithm written in F77 (tsaclean.for)
! .PURPOSE  
!
! execute the command :
!
! TS2PERIGRAM out = inframes x_intv [t_intv] [pre] [clean-flag[out]]
!  where
!      inframes  :  catalog.cat (time series)
!      outimage  :  prefix for outimages (all in wavelength - frequency frame)
!			outimagePER  	: periodogram
!			outimagePERQ 	: quality flags (1-dim)
!			outimagePOW     : powerspectrum
!			outimageWIN     : window function  (1-dim)
!      x_interval: x_low,x_high of x interval 
!      t_interval: cat_entry_start,cat_entry_stop, defaulted to all 
!      pre       : prewhitening with 1/obs.time [Y/N] (only in the first step)
!      clean-flag out : if flag = N 
!                               = Y then a CLEAN-like procedure will be started
!	                        = I then the next iteration for CLEAN will be
!                                   started, the result of the previous
!                                   iteration is used as input
!
DEFINE/PAR P1 ? IMA "Enter prefix for outframes (x - freq frame) : "
DEFINE/PAR P3 ? C   "Enter catalog.cat with 1-dim frames    : "
DEFINE/PAR P4 ? N   "Enter x interval xlo,xhi: "
DEFINE/PAR P5 0,0 N "Enter valid catalog interval tlo,thi : "
DEFINE/PAR P6 N   C "Prewhiten flag [Y/N] : "
DEFINE/PAR P7 N   C "CLEAN flag [Y/N/I]: "
DEFINE/PAR P8 Y   C "Renorm flag [Y/N]: "
! 
define/local prewhit/c/1/1 {p6}
define/local clean/c/1/1  {p7}
define/local cols/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0
define/local iter/i/1/1 0
define/local niter/i/1/1 0
define/local exist/i/1/1 0
define/local pos/i/1/1 0
define/local vel/d/1/1 0.
define/local lstart/d/1/1 0.
define/local lstep/d/1/1 0.
define/local lnpix/i/1/1 0
define/local tsstart/d/1/1 0.
define/local tsend/d/1/1 0.
define/local tsstep/d/1/1 0.
define/local tsnsteps/i/1/1 0
define/local ndata/i/1/1 0
define/local method/c/1/3 " " all
define/local prefreq/r/1/1 0.
define/local los/r/1/1 0.
define/local maxpow/d/1/1 0.
!
!define/local alpha/r/1/1 0.05 ! probability for level of significance,   1 sigma
define/local alpha/r/1/1 0.005 ! probability for level of significance,   3 sigma
define/local fititer/i/1/1 15  ! number of iterations for sinefit

define/local extlen1/i/1/1 0
define/local extlen2/i/1/1 0

extlen1 = m$index(mid$types(1:8)," ")-1
extlen2 = m$index(mid$types(9:16)," ")+8

!
write/key method/c/1/3 SCA
write/out "---------------------------------------------"
write/out " method {method} is used for test statistics " 
write/out "---------------------------------------------"
!
!set/context tsa
! order MUST be equal 1 for CLEAN !
ordertsa = 1
starttsa = 0.0
!
set/gra pmode=1
!
if "{clean}" .ne. "I" then
!
! convert time series in table with columns for each x step
!
! table/ts middummtab = {p3} {p4} {p5} Y N 
 table/ts middummtab = {p3} {p4} {p5} {p8} N 
!
 copy/tt &tab :jd24 ? :time
 normalize/tsa &tab ? :time
 crea/col &tab :value " " G8.3 R*8
 if "{clean}" .ne. "N" then
  copy/tab &tab {p1}CLEAN{iter}
  niter = 1
 endif 
!
else
! find number of last iteration
 iter = 1
next:
 write/key test/c/1/20 "{p1}CLEAN{iter}.{mid$types(9:{extlen2})}"
 exist =  M$EXIST (test)
 if {exist} .eq. 1 then
  iter = iter + 1
  goto next 
 endif
! use table from last iteration
 iter = iter-1
 copy/tab {p1}CLEAN{iter} &tab        
 niter = iter+1
 write/out "---------------------------------------------"
 write/out "        Iteration step # {iter}              " 
 write/out "---------------------------------------------"
endif
!
! create the x - frequency frame for the results of transformation
!
write/out "---------------------------------------------"
copy/dk &tab.{mid$types(9:{extlen2})} lnpix/i/1/1 cols
write/out "{cols} x data points found"
copy/dk &tab.{mid$types(9:{extlen2})} tblcontr/i/4/4 ndata
write/out "{ndata} time data points found"
write/out "---------------------------------------------"
!
! compute level of significance
!
if method(1:3) .eq. "AOV" then
 inputi(1) = ordertsa - 1
 inputi(2) = ndata - ordertsa
 inputr(1) = {alpha}
elseif method(1:3) .eq. "SCA" then
 inputi(1) = 2
 inputi(2) = ndata
 inputr(1) = {alpha}
endif
run TSEXEC:fdistpp
!
set/format f5.2
outputr(2) = (1.-inputr(1))*100.
write/out
write/out "   If NC = 1 then the level of significance = {outputr(1)}" 
write/out "   (with a probabilty of {outputr(2)} percent)"
set/format
los = outputr(1)
write/descr &tab.{mid$types(9:{extlen2})} los/r/1/1 {los}
write/out
!
! set parameters for prewhitening
!
if "{prewhit}" .eq. "Y" .and. "{clean}" .ne. "I" then
 stat/tab &tab :time
 prefreq = 1./(outputr(2)-outputr(1))
 write/descr &tab.{mid$types(9:{extlen2})} prefreq/r/1/1 {prefreq}
 write/out
 write/out "Prewhitening with f={prefreq}"
 write/out
endif
!
write/out
write/out "The following settings are used: "
show/tsa
write/out
!
! create result frames and tables 
!
if "{clean}" .ne. "N"  then
!
! create table for fitted frequencies, phases and maximum power
!
 crea/tab &tabf 20 1000
 crea/col &tabf :vel " " D12.4 R*8
 crea/col &tabf :freq " " D12.4 R*8
 crea/col &tabf :dfreq " " D12.4 R*8
 crea/col &tabf :phmt " " D12.4 R*8
 crea/col &tabf :dphmt " " D12.4 R*8
!columns for fit coefficients
 j = 2*ordertsa+1
 do i = 1 {j}
  crea/col &tabf :c{i} " " D12.4 R*8   
 enddo
!column for max power in the max peak
 crea/col &tabf :maxpow " " D12.4 R*8
endif
!
! create result frames in double precision
!
copy/dk &tab.{mid$types(9:{extlen2})} lstart/d/1/1 lstart
copy/dk &tab.{mid$types(9:{extlen2})} lstep/d/1/1 lstep
copy/dk &tab.{mid$types(9:{extlen2})} lnpix/i/1/1 lnpix
!
lstart = lstart+(lnpix-1)*lstep
lstep = -1*lstep
crea/ima &xx 2,{nsteps},{lnpix} {starttsa},{lstart},{steptsa},{lstep}
copy/ii &xx &res R8 N
copy/ii &xx &pow R8 D
!
!copy/dd &tab.{mid$types(9:{extlen2})} lamc/d/1/1 &res lamc/d/1/1
!copy/dd &tab.{mid$types(9:{extlen2})} lamc/d/1/1 &pow lamc/d/1/1
!
! compute the 2D power spectrum/periodogram and apply CLEAN if wanted
!
run TSEXEC:tsaclean
!
!
if "{clean}" .ne. "N" then
 write/out "------------------------------------------------------------------"
!
! save table with CLEANed columns
!
 copy/tab &tab {p1}CLEAN{niter}
 write/out " CLEANed columns are stored in {p1}CLEAN{niter}"
!
! save table with fitted frequencies and phases
!
 comp/tab &tabf :P = 1/:freq
 comp/tab &tabf :dP = :P*:dfreq/:freq
! comp/tab &tabf :t0 = :phmt/:freq/6.2832
! comp/tab &tabf :dt0 = :dphmt/:freq/6.2832
 comp/tab &tabf :maxA = SQRT(:maxpow)
 sele/tab &tabf :freq.ne.NULL
 copy/tab &tabf {p1}FIT{niter}
 if {niter} .eq. 1 then
  copy/tab {p1}FIT{niter} {p1}FITm
 else
  copy/tab {p1}FITm &tmp
  merge/tab &tmp {p1}FIT{niter} {p1}FITm   
 endif
 write/out " Fitted freq. and phases at mean time are stored in {p1}FIT{niter}"
 write/out " All so far computed {p1}FIT tables a merged in {p1}FITm          "
 if "{clean}" .eq. "Y" then
  write/out 
  write/out " Next iteration by restarting the command with I instead of Y"
 endif 
 write/out "------------------------------------------------------------------"
endif
!
rotate/counter &res {p1}PER{iter}
rotate/counter &pow {p1}POW{iter}
!
displ/per {p1}POW{iter} ? ? "Wavelength" 
!
! some comments on the resulting frames
!
if "{clean}" .eq. "N" then
 write/out
 write/out "FINAL STEPS TO DO:"
 write/out
 write/out "1. {p1}PER0000 must be normalized to a continuum of unity now: "
 write/out "  - Measure the continuum level NC  "
 write/out "    If it's different from unity the data are correlated " 
 write/out "  - Divide {p1}PER0000 by NC: 
 write/out "    COMP/IMA {p1}PER0000n = {p1}PER0000/NC "
 write/out
!
 if method(1:3) .eq. "AOV" then
  write/out "2. Find level of significance by looking up percentage points of "
  write/out "   the F-distribution for F({ordertsa}-1,{ndata}/NC-{ordertsa})  "
  write/out "   You may use the command @@ TSPROC:fdistpp m n alpha          "
  inputi(1) = ordertsa - 1
  inputi(2) = ndata - ordertsa
  inputr(1) = {alpha}
 elseif method(1:3) .eq. "SCA" then
  write/out "2. Find level of significance by looking up percentage points of "
  write/out "   the F-distribution for F(2,{ndata}/NC)  "
  write/out "   You may use the command @@ TSPROC:fdistpp m n alpha          "
  inputi(1) = 2
  inputi(2) = ndata
  inputr(1) = {alpha}
 endif
! 
 run TSEXEC:fdistpp
!
 set/format F5.2
 outputr(2) = (1.-inputr(1))*100.
 write/out
 write/out "   If NC = 1 then the level of significance = {outputr(1)}" 
 write/out "   (with a probabilty of {outputr(2)} percent)"
 set/format
endif
!
dele/temp
end:



