! @@ ts2perigram.prg
! ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!
! TS2PERIGRAM.PRG
! A.KAUFER - LSW Heidelberg
! AK            940221  first version
!               940803  CLEAN-like algorithm introduced
! .PURPOSE  
!
! execute the command :
!
! TS2PERIGRAM out = inframes x_intv [t_intv] [vstep] [method] [clean-flag[out]]
!  where
!      inframes  :  catalog.cat (time series)
!      outimage  :  prefix for outimages (all in wavelength - frequency frame)
!			outimagePERIGR  : periodogram
!			outimagePERIGRQ : quality flags (1-dim)
!			outimagePOW     : powerspectrum
!			outimageWIN     : window function  (1-dim)
!      x_interval: lambda_center,v_low,v_high of velocity interval 
!      t_interval: cat_entry_start,cat_entry_stop, defaulted to all 
!      vstep : velocity step for rebinning
!      method           : test statistics for periodogram
!                           AOV
!			    SCARGLE 
!      clean-flag out : if flag = N 
!                               = Y then a CLEAN-like procedure will be started
!	                        = I then the next iteration for CLEAN will be
!                                   started, the result of the previous
!                                   iteration is used as input
!
DEFINE/PAR P1 ? IMA "Enter prefix for outframes (veloc - freq frame) : "
DEFINE/PAR P3 ? C   "Enter catalog.cat with 1-dim frames    : "
DEFINE/PAR P4 ? N   "Enter central wavelength and velocity interval lamc,vlo,vhi: "
DEFINE/PAR P5 0,0 N "Enter valid catalog interval tlo,thi : "
DEFINE/PAR P6 5   N "Enter valid velocity step : "
DEFINE/PAR P7 SCA C "Test statistics [AOV/SCARGLE] : "
DEFINE/PAR P8 N   C "CLEAN flag [Y/N/I]: "
! 
define/local cols/i/1/1 0
define/local iii/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0
define/local iter/i/1/1 0
define/local niter/i/1/1 0
define/local exist/i/1/1 0
define/local pos/i/1/1 0
define/local vel/d/1/1 0.
define/local vstart/d/1/1 0.
define/local vstep/d/1/1 0.
define/local vnpix/i/1/1 0
define/local tsstart/d/1/1 0.
define/local tsend/d/1/1 0.
define/local tsstep/d/1/1 0.
define/local tsnsteps/i/1/1 0
define/local ndata/i/1/1 0
define/local method/c/1/3 " " all
define/local old_ph/d/1/1 0.
define/local ph_diff/d/1/1 0.
define/local ph_off/d/1/1 0.
define/local prefreq/r/1/1 0.
define/local los/r/1/1 0.
define/local los2/r/1/1 0.
define/local maxpow/d/1/1 0.
!
define/local fititer/i/1/1 15  ! number of iterations for sinefit
!
write/key method/c/1/3 {p7}
write/out "---------------------------------------------"
write/out " method {method} is used for test statistics " 
write/out "---------------------------------------------"
!
!set/context tsa
! order MUST be equal 1 for CLEAN !
ordertsa = 1
starttsa = 0.0
!
set/gra pmode=1
!
if "{p8}" .ne. "I" then
!
! convert time series in table with columns for each velocity step
!
 vtable/ts middummtab = {p3} {p4} {p5} {p6} Y Y 
!
 copy/tt &tab :jd24 ? :time
 normalize/tsa &tab ? :time
 crea/col &tab :value " " G8.3 R*8
 if "{p8}" .ne. "N" then
  copy/tab &tab {p1}CLEAN{iter}
  niter = 1
 endif 
!
else
! find number of last iteration
 iter = 1
next:
 write/key test/c/1/20 "{p1}CLEAN{iter}.{mid$types(9:{extlen2})}"
 exist =  M$EXIST (test)
 if {exist} .eq. 1 then
  iter = iter + 1
  goto next 
 endif
! use table from last iteration
 iter = iter-1
 copy/tab {p1}CLEAN{iter} &tab        
 niter = iter+1
endif
!
! create the velocity - frequency frame for the results of transformation
!
write/out "---------------------------------------------"
copy/dk &tab.{mid$types(9:{extlen2})} vnpix/i/1/1 cols
write/out "{cols} velocity data points found"
copy/dk &tab.{mid$types(9:{extlen2})} tblcontr/i/4/4 ndata
write/out "{ndata} time data points found"
write/out "---------------------------------------------"
!
! compute level of significance
!
if method(1:3) .eq. "AOV" then
INPUTI(1) = ordertsa - 1
INPUTI(2) = ndata - ordertsa
INPUTR(1) = 0.05
elseif method(1:3) .eq. "SCA" then
INPUTI(1) = 2
INPUTI(2) = ndata
INPUTR(1) = 0.05
endif
RUN TSEXEC:FDISTPP
!
SET/FORMAT F5.2
OUTPUTR(2) = (1.-INPUTR(1))*100.
WRITE/OUT
WRITE/OUT "   If NC = 1 then the level of significance = {OUTPUTR(1)}" 
WRITE/OUT "   (with a probabilty of {OUTPUTR(2)} percent)"
SET/FORMAT
los = outputr(1)
!
! set parameters for transformation
!
copy/tt &tab :l0001 ? :value
normalize/tsa &tab ? :value
!
! parameters for prewhitening
!
stat/tab &tab :time
prefreq = 1./(outputr(2)-outputr(1))
write/descr &tab.{mid$types(9:{extlen2})} Prefreq/r/1/1 {prefreq}
write/out "prewhitening with f={prefreq}"
!
!band/tsa &tab
!
!starttsa = 0.0
!steptsa = .25E-03
!nsteps = 700
!
tsstep   = steptsa
tsstart  = starttsa
tsnsteps = nsteps
tsend    = tsstart+tsnsteps*tsstep
write/out
write/out "The follwoing settings are used: "
show/tsa
!
if "{p8}" .ne. "N"  then
!
! create table for fitted frequencies, phases
!
 crea/tab &tabf 20 1000
 crea/col &tabf :vel " " D12.4 R*8
 crea/col &tabf :freq " " D12.4 R*8
 crea/col &tabf :dfreq " " D12.4 R*8
 crea/col &tabf :phmt " " D12.4 R*8
 crea/col &tabf :dphmt " " D12.4 R*8
!columns for fit coefficients
 j = 2*ordertsa+1
 do i = 1 {j}
  crea/col &tabf :c{i} " " D12.4 R*8   
 enddo
 crea/col &tabf :maxpow " " D12.4 R*8
endif
!
!
copy/dk &tab.{mid$types(9:{extlen2})} vstart/d/1/1 vstart
copy/dk &tab.{mid$types(9:{extlen2})} vstep/d/1/1 vstep
copy/dk &tab.{mid$types(9:{extlen2})} vnpix/i/1/1 vnpix
!
vstart = vstart+(vnpix-1)*vstep
vstep = -1*vstep
crea/ima &res 2,{nsteps},{vnpix} {starttsa},{vstart},{steptsa},{vstep}
copy/dd &tab.{mid$types(9:{extlen2})} lamc/d/1/1 &res lamc/d/1/1
crea/ima &power = &res
copy/dd &tab.{mid$types(9:{extlen2})} lamc/d/1/1 &power lamc/d/1/1
!
! Loop over the columns of the table and transform by AOV/SCARGLE
!
do iii = 1 {cols} 1
 vel = vstart+(cols-iii)*vstep
 write/out
 write/out processing column {iii} of {cols} at {vel} km/s
!
!transform column
!
 copy/tt &tab :l{iii} ? :value
 dele/descr &tab.{mid$types(9:{extlen2})} HISTORY 
 normalize/tsa &tab ? :value
! show/tsa
!
! Prewhiten data (NOT recommended!!)
!
 if "{p8}" .eq. "Y" then
  sinefit/tsa &tab &tabcl {prefreq} 1 1
  copy/tab &tabcl &tab
 endif
!
! compute test statistics 
!
 {p7}/tsa &tab &perigr ? ? ? ?
 plot &perigr
 over/line 1 {tsstart},{los} {tsend},{los}
!
! compute power spectrum
!
 power/tsa &tab &pow ? ? ? 
 plot &pow
!
! CLEAN algorithm 
!
if "{p8}" .ne. "N" then
!
! find freq with max power, fit sinusoid -> freq, phase at mean time
!
 &tabf,:vel,@{iii} = {vel}
!
! clean only if significant peaks exist 
 stat/ima &perigr
 los2 = 1.2*los
 if {outputr(2)} .gt. {los2} then
!
!  outputr(5) = -1
!  write/out
!  write/out "Select the frequency to fit by graphics cursor : "
!  write/out
!  center/gauss GCURSOR
!  if {outputr(5)} .GT. -1 then
!  @@ TSPROC:tsawdt &pow   {outputr(5)} {outputr(3)} 
!  width/tsa &pow
  @@ TSPROC:tsawdt &pow
  maxpow = outputd(11)
  write/out maxpow = {maxpow}
!
!  sinfit/tsa &tab {p1}CLEAN{niter} ? ? 5
   @@ TSPROC:tsasin &tab &tabCLEAN ? ? {fititer}
!
!  store back the cleaned column
   copy/tt &tabCLEAN :value &tab :l{iii} 
!
!  write results (frequency, phase at mean time) to table
   copy/kd ordertsa/i/1/1  &tabf.{mid$types(9:{extlen2})} order/i/1/1
   i = 2*ordertsa+1
   i = i+1
   &tabf,:freq,@{iii} =  {outputd({i})}
   i = i+1
   &tabf,:dfreq,@{iii} =  {outputd({i})}
   i = i+1
!  correct for phase jumps 
!   if {iii} .gt. 1 then
!    ph_diff = outputd({i})-old_ph
!    if {ph_diff} .gt. 0.5 then
!     ph_off = ph_off+1.0
!    endif
!    &tabf,:phmt,@{iii} =  {outputd({i})}-{ph_off}
!    old_ph = {outputd({i})}
!   else
    &tabf,:phmt,@{iii} =  {outputd({i})}
!    old_ph = {outputd({i})}
!   endif
   i = i+1
   &tabf,:dphmt,@{iii} =  {outputd({i})}
!  write fit coefficients
   j = 2*ordertsa+1
   do i = 1 {j}
    &tabf,:c{i},@{iii} =  {outputd({i})}
   enddo
   &tabf,:maxpow,@{iii} = {maxpow}
! endif
!
  steptsa = tsstep 
  starttsa = tsstart  
  nsteps = tsnsteps
!
! inspect result of cleaning 
  power/tsa &tabCLEAN &xx ? ? ? 
  set/gra color=4
  over &xx
  set/gra color=1
!
 endif
endif
! 
! insert to result frames
!
 pos = cols-iii+1
 write/out inserting to row {pos}
 extract/ima &row = &perigr[<,@1:>,@1]
 copy/dk &row start/d/1/1 rstart/d/1/1
 insert/ima &row &res {rstart},@{pos}
 extract/ima &row = &pow[<,@1:>,@1]
 copy/dk &row start/d/1/1 rstart/d/1/1
 insert/ima &row &power {rstart},@{pos}
!
 rotate/counter &power &prelim
 displ/per &prelim ? ? "Velocity"
enddo
!
if "{p8}" .ne. "N" then
write/out "-------------------------------------------------------------------"
! save table with CLEANED columns
copy/tab &tab {p1}CLEAN{niter}
write/out " CLEANED columns are stored in {p1}CLEAN{niter}"
! save table with fitted frequencies and phases
comp/tab &tabf :phmt = :phmt+{ph_off}
comp/tab &tabf :P = 1/:freq
comp/tab &tabf :dP = :P*:dfreq/:freq
comp/tab &tabf :t0 = :phmt/:freq/6.2832
comp/tab &tabf :dt0 = :dphmt/:freq/6.2832
copy/tab &tabf {p1}FIT{niter}
write/out " Fitted freq. and phases at mean time are stored in {p1}FIT{niter}"
write/out "-------------------------------------------------------------------"
endif
!
extract/ima {p1}PERIGRQ{iter} = &perigr[<,@2:>,@2]
!plo {p1}PERIGRQ{iter}
extract/ima {p1}WIN{iter}  = &pow[<,@2:>,@2]
!plo {p1}WIN{iter}
rotate/counter &res    {p1}PERIGR{iter}
rotate/counter &power  {p1}POW{iter}
displ/per {p1}POW{iter}
!
if "{p8}" .eq. "N" then
write/out
write/out "FINAL STEPS TO DO:"
write/out
write/out "1. {p1}PERIGR must be normalized to a continuum of unity now: "
write/out "  - Measure the continuum level NC  "
write/out "    If it's different from unity the data are correlated " 
write/out "  - Divide {p1}PERIGR by NC: COMP/IMA {p1}PERIGRn = {p1}PERIGR/NC "
write/out
!
if method(1:3) .eq. "AOV" then
write/out "2. Find level of significance by looking up percentage points of " 
write/out "   the F-distribution for F({ordertsa}-1,{ndata}/NC-{ordertsa})  "
write/out "   You may use the command @@ TSPROC:fdistpp m n alpha          "
INPUTI(1) = ordertsa - 1
INPUTI(2) = ndata - ordertsa
INPUTR(1) = 0.05
elseif method(1:3) .eq. "SCA" then
write/out "2. Find level of significance by looking up percentage points of " 
write/out "   the F-distribution for F(2,{ndata}/NC)  "
write/out "   You may use the command @@ TSPROC:fdistpp m n alpha          "
INPUTI(1) = 2
INPUTI(2) = ndata
INPUTR(1) = 0.05
endif
! 
RUN TSEXEC:FDISTPP
!
SET/FORMAT F5.2
OUTPUTR(2) = (1.-INPUTR(1))*100.
WRITE/OUT
WRITE/OUT "   If NC = 1 then the level of significance = {OUTPUTR(1)}" 
WRITE/OUT "   (with a probabilty of {OUTPUTR(2)} percent)"
SET/FORMAT
endif
!
dele/temp
end:


