      PROGRAM CHISQRDISTPP
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.NAME
C      CHISQRDISTPP.FOR                            VERSION 1.0
C
C.AUTHOR
C      A. KAUFER (LSW HD) VERSION 1.0	19-OCT-1994
C
C.MODIFICATIONS
C
C.PURPOSE  Calculate percentage points of chi squared distribution
C	   i.e. calculate the level of signifcance for given degree of 
C	        freedom n and given probabilty alpha for the level of
C		significance 
C
C          \alpha = GAMMP(n/2,\chi^2/2)
C
C.INPUT
C      n : degree of freedom                         INPUTI(1:1)
C      alpha : probabilty for the significance level INPUTR(1:1)
C
C.OUTPUT
C	OUTPUTR(1)  significance level
C
C---------------------------------------------------------------
C
      IMPLICIT NONE
C
      REAL ALPHA,C,CSTEP,Q
      INTEGER N
      INTEGER IACTV,ISTAT,KUN(1),KNUL
C
      DATA CSTEP /0.01/
C
      CALL STSPRO('CFDISTPP')
C
      CALL STKRDR('INPUTR',1,1,IACTV,ALPHA,KUN,KNUL,ISTAT)
      CALL STKRDI('INPUTI',1,1,IACTV,N,KUN,KNUL,ISTAT)
C
      C = 0.
 10   Q = CDIST(C,N)
c      write(6,*) c,q,n
      IF (Q.LE.ALPHA) GOTO 20
      C = C+CSTEP
      GOTO 10
C
 20   CALL STKWRR('OUTPUTR',C,1,1,KUN,ISTAT)
C
      CALL STSEPI
C
      END
C
C
      FUNCTION CDIST(C,N)
      INTEGER N
      REAL C,A,X
      A = N/2.
      X = C/2.
      CDIST = GAMMQ(A,X)      
      RETURN
      END
C
C
      FUNCTION GAMMQ(A,X)
      IF(X.LT.0..OR.A.LE.0.)PAUSE
      IF(X.LT.A+1.)THEN
        CALL GSER(GAMSER,A,X,GLN)
        GAMMQ=1.-GAMSER
      ELSE
        CALL GCF(GAMMCF,A,X,GLN)
        GAMMQ=GAMMCF
      ENDIF
      RETURN
      END
C
C
      SUBROUTINE GSER(GAMSER,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      IF(X.LE.0.)THEN
        IF(X.LT.0.)PAUSE
        GAMSER=0.
        RETURN
      ENDIF
      AP=A
      SUM=1./A
      DEL=SUM
      DO 11 N=1,ITMAX
        AP=AP+1.
        DEL=DEL*X/AP
        SUM=SUM+DEL
        IF(ABS(DEL).LT.ABS(SUM)*EPS)GO TO 1
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMSER=SUM*EXP(-X+A*LOG(X)-GLN)
      RETURN
      END
C
C
      SUBROUTINE GCF(GAMMCF,A,X,GLN)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      GLN=GAMMLN(A)
      GOLD=0.
      A0=1.
      A1=X
      B0=0.
      B1=1.
      FAC=1.
      DO 11 N=1,ITMAX
        AN=FLOAT(N)
        ANA=AN-A
        A0=(A1+A0*ANA)*FAC
        B0=(B1+B0*ANA)*FAC
        ANF=AN*FAC
        A1=X*A0+ANF*A1
        B1=X*B0+ANF*B1
        IF(A1.NE.0.)THEN
          FAC=1./A1
          G=B1*FAC
          IF(ABS((G-GOLD)/G).LT.EPS)GO TO 1
          GOLD=G
        ENDIF
11    CONTINUE
      PAUSE 'A too large, ITMAX too small'
1     GAMMCF=EXP(-X+A*ALOG(X)-GLN)*G
      RETURN
      END
C
C
      FUNCTION GAMMLN(XX)
      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER
      DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *    -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      GAMMLN=TMP+LOG(STP*SER)
      RETURN
      END

