      PROGRAM FDISTPP
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.NAME
C      FDISTPP.FOR                            VERSION 1.0
C
C.AUTHOR
C      A. KAUFER (LSW HD) VERSION 1.0	26-FEB-1994
C
C.MODIFICATIONS
C
C.PURPOSE  Calculate percentage points of F distribution
C	   i.e. calculate the level of signifcance for given degrees of 
C	        freedom m,n and given probabilty alpha for the level of
C		significance 
C
C \alpha = \int_0^F \frac{\Gamma\left(\frac{m+n}{2}\right)}
C                        {\Gamma\left(\frac{m}{2}\right)
C			  \Gamma\left(\frac{n}{2}\right)}
C	            m^{m/2} n^{n/2} x^{(m-2)/2} (n+mx)^{-(m+n)/2} {\rm d}x
C
C.INPUT
C      m : degree of freedom of the numerator        INPUTI(1:1)
C      n : degree of freedom of the denominator      INPUTI(2:2)
C      alpha : probabilty for the significance level INPUTR(1:1)
C
C.OUTPUT
C	OUTPUTR(1)  significance level
C
C---------------------------------------------------------------
C
      IMPLICIT NONE
C
      REAL ALPHA,F,FSTEP,Q
      INTEGER M,N
      INTEGER IACTV,ISTAT,KUN(1),KNUL
C
      DATA FSTEP /0.001/
C
      CALL STSPRO('FDISTPP')
C
      CALL STKRDR('INPUTR',1,1,IACTV,ALPHA,KUN,KNUL,ISTAT)
      CALL STKRDI('INPUTI',1,1,IACTV,M,KUN,KNUL,ISTAT)
      CALL STKRDI('INPUTI',2,1,IACTV,N,KUN,KNUL,ISTAT)
C
      F = 0.
 10   Q = FDIST(F,M,N)
      IF (Q.LE.ALPHA) GOTO 20
      F = F+FSTEP
      GOTO 10
C
 20   CALL STKWRR('OUTPUTR',F,1,1,KUN,ISTAT)
C
      CALL STSEPI
C
      END
C
C
      FUNCTION FDIST(F,M,N)
      INTEGER M,N
      REAL F,A,B,X
      A = N/2.
      B = M/2.
      X = N/(N+M*F)
      FDIST = BETAI(A,B,X)      
      RETURN
      END
C
C
      FUNCTION BETAI(A,B,X)
      IF(X.LT.0..OR.X.GT.1.)PAUSE 'bad argument X in BETAI'
      IF(X.EQ.0..OR.X.EQ.1.)THEN
        BT=0.
      ELSE
        BT=EXP(GAMMLN(A+B)-GAMMLN(A)-GAMMLN(B)
     *      +A*ALOG(X)+B*ALOG(1.-X))
      ENDIF
      IF(X.LT.(A+1.)/(A+B+2.))THEN
        BETAI=BT*BETACF(A,B,X)/A
        RETURN
      ELSE
        BETAI=1.-BT*BETACF(B,A,1.-X)/B
        RETURN
      ENDIF
      END
C
      FUNCTION GAMMLN(XX)
      REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER
      DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *    -1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
      DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
      X=XX-ONE
      TMP=X+FPF
      TMP=(X+HALF)*LOG(TMP)-TMP
      SER=ONE
      DO 11 J=1,6
        X=X+ONE
        SER=SER+COF(J)/X
11    CONTINUE
      GAMMLN=TMP+LOG(STP*SER)
      RETURN
      END
C
      FUNCTION BETACF(A,B,X)
      PARAMETER (ITMAX=100,EPS=3.E-7)
      AM=1.
      BM=1.
      AZ=1.
      QAB=A+B
      QAP=A+1.
      QAM=A-1.
      BZ=1.-QAB*X/QAP
      DO 11 M=1,ITMAX
        EM=M
        TEM=EM+EM
        D=EM*(B-M)*X/((QAM+TEM)*(A+TEM))
        AP=AZ+D*AM
        BP=BZ+D*BM
        D=-(A+EM)*(QAB+EM)*X/((A+TEM)*(QAP+TEM))
        APP=AP+D*AZ
        BPP=BP+D*BZ
        AOLD=AZ
        AM=AP/BPP
        BM=BP/BPP
        AZ=APP/BPP
        BZ=1.
        IF(ABS(AZ-AOLD).LT.EPS*ABS(AZ))GO TO 1
11    CONTINUE
      PAUSE 'A or B too big, or ITMAX too small'
1     BETACF=AZ
      RETURN
      END


