      SUBROUTINE INTFRB2(VAL,NPL,IMAGE,SX,XS,NCUR,NDEG,             
     2                  DELTAX,CENTER,XPC)
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.AUTHOR:   F. Murtagh, ST-ECF                                                 
C.VERSION:  860321 FM Creation
C.VERSION:  880607 RHW Restructured and recoded for portable MIDAS
C.VERSION:  900430 RHW Make code similar to INTFRI 
C.VERSION:  900430 RHW Store more output in the keyword OUTPUTR
C.VERSION:  900629 RHW Integration over subpixel included
C.VERSION:  910115 RHW IMPLICIT NONE added
C----------------------------------------------------------------------------
      IMPLICIT  NONE
      INTEGER   MAXC
      PARAMETER (MAXC=100)
C
      INTEGER   NPL
      REAL      VAL(NPL)
      REAL      IMAGE(4)          
      REAL      SX
      REAL      XS
      INTEGER   NCUR
      INTEGER   NDEG
      REAL      DELTAX,CENTER
      REAL      XPC(MAXC)
C
      INTEGER   MAXT
      PARAMETER (MAXT=100000)
      INTEGER   ISTAT
      INTEGER   KUN
      INTEGER   I, I2, INNER
      INTEGER   NP1, NP2, NP3, NP4
      INTEGER   NPL1, NPL2, NPL3, NPT
      REAL      AITKEN, ALINE, ALINEB, ALINER, ALINT
      REAL      ACONT, ALICO, ACONTB, ALICOB, ACONTR, ALICOR
      REAL      ACONT3, ACONT4, ALICO3, ALICO4
      REAL      AEQW,AEQWB,AEQWR
      REAL      DX3, DX4
      REAL      FLXFRC
      REAL      OUTP(12)                                            
      REAL      RC3, RC4
      REAL      SX3, SX4
      REAL      WORK(MAXT)
      REAL      XARG
      REAL      Y
      REAL      XC3, XC4
      REAL      XRANG, XRANGB, XRANGR
      REAL      YPC(MAXC) 
C
C *** start of executable code **********************************************
C     Now do the looping for cursers and interpolations.
C     NCUR positions are asked for.
      NPL1  = INT(MIN(IMAGE(1),IMAGE(2)))
      NPL2  = INT(MAX(IMAGE(1),IMAGE(2))) 
      NPL3  = 1 
      NPT   = ABS(NPL1-NPL2) + 1 
C
C *** loop through cursor position and determine nearest pixels
      DO 10 I = 1, NCUR 
         NP1 = NINT((XPC(I)-DELTAX-XS)/SX) + 1                             
         NP2 = NINT((XPC(I)+DELTAX-XS)/SX) + 1                             
         IF (NP1.LT.NPL1 .OR. NP1.GT.NPL2 .OR.
     2       NP2.LT.NPL1 .OR. NP2.GT.NPL2) THEN
             CALL STTPUT('*** FATAL: Cursor position(s) outside'//
     2                   ' frame; try again', ISTAT)
             CALL STSEPI
         ENDIF
         Y   = 0.0
         DO 20 I2 = NP1, NP2 
            Y = Y + VAL(I2)
   20    CONTINUE
         Y     = Y/FLOAT(NP2-NP1+1)
         YPC(I) = Y 
   10 CONTINUE 
      CALL SORT2(XPC,YPC,NCUR)
C
C *** Determine effective range of cursor positions
      NP1 = NINT((XPC(1)-XS)/SX) + 1                    ! first cursor position
      NP2 = NINT((XPC(NCUR)-XS)/SX) + 1                 ! last cursor position
C
C *** Do the interpolation here
      IF (NDEG.LT.2) THEN                              !compute the continuum
         DO 30 I = NP1,NP2
            XARG    = XS + FLOAT(I-1)*SX
            WORK(I) = ALINT(XPC,YPC,NCUR,XARG,1,NCUR)
   30    CONTINUE
     
       ELSE
         DO 40 I = NP1,NP2
            XARG    = XS + FLOAT(I-1)*SX             
            WORK(I) = AITKEN(XPC,YPC,NCUR,XARG,1,NCUR,NDEG)
   40    CONTINUE
      ENDIF
C
C *** Do the integration 
C     FIRST compute the flux density in the first bin.
C     The bin limits are the input cursor position and the middle between
C     the first and second image pixel. We compute the interpolated values
C     at these points and make the integral in the first bin.
      INNER  = NCUR/2                               ! first inner pixel
      XC3    = XPC(INNER)                           ! real cursor position
      RC3    = (XC3-XS)/SX+1.0                      ! real cursor pixel
      NP3    = NINT((XC3-XS)/SX)+1                  ! nearest image pixel
      DX3    = FLOAT(NP3)-RC3
      SX3    = (DX3+0.5)*SX
      ACONT3 = SX3*WORK(NP3)
      ALICO3 = SX3*VAL(NP3)
C
C *** SECOND compute the flux density in the last bin.
C     The bin limits are the last cursor position and the middle between
C     the last and one but last image pixel. We compute the interpolated 
C     values at these points and make the integral in the last bin.
C     and then do the integration.
      XC4    = XPC(INNER+1)
      RC4    = (XC4-XS)/SX+1.0
      NP4    = NINT((XC4-XS)/SX)+1
      DX4    = RC4-FLOAT(NP4)
      SX4    = (DX4+0.5)*SX 
      ACONT4 = SX4*WORK(NP4)
      ALICO4 = SX4*VAL(NP4)
C
      NPC    = NINT((CENTER-XS)/SX)+1       ! nearest image pixel of center
C
C *** do the pixel summation for continuum and line
      ACONT  = 0.
      ALICO  = 0.
      ACONTB  = 0.
      ALICOB  = 0.
      ACONTR  = 0.
      ALICOR  = 0.
      DO 50 I = NP3+1,NP4-1                                                    
          ACONT = WORK(I)*SX + ACONT                                          
          ALICO = VAL(I)*SX + ALICO
          IF (I.LE.NPC) THEN
           ACONTB = WORK(I)*SX + ACONTB     
           ALICOB = VAL(I)*SX + ALICOB
          ELSE
           ACONTR = WORK(I)*SX + ACONTR                                        
           ALICOR = VAL(I)*SX + ALICOR
          ENDIF
   50 CONTINUE                  
      ACONT    = ACONT + ACONT3 + ACONT4
      ALICO    = ALICO + ALICO3 + ALICO4
      XRANG    = XPC(INNER+1)-XPC(INNER)
c      write(6,*) xrang,xpc(inner+1),xpc(inner)
      ALINE    = ALICO - ACONT                                      
      AEQW     = -1.0*(ALINE/ACONT)*XRANG                          
      FLXFRC   = ALINE/ACONT

      ACONTB    = ACONTB + ACONT3
      ALICOB    = ALICOB + ALICO3
      XRANGB    = CENTER-XPC(INNER)
c      write(6,*) center,xpc(inner),xrangb
      ALINEB    = ALICOB - ACONTB                                      
      AEQWB     = -1.0*(ALINEB/ACONTB)*XRANGB                          

      ACONTR    = ACONTR + ACONT4
      ALICOR    = ALICOR + ALICO4
      XRANGR    = XPC(INNER+1)-CENTER
c      write(6,*) center,xpc(inner+1),xrangr
      ALINER    = ALICOR - ACONTR                                      
      AEQWR     = -1.0*(ALINER/ACONTR)*XRANGR                          
C ***
C
      OUTP(1)  = RC3                                      ! first NUMBER
      OUTP(2)  = RC4                                      ! last pixel
      OUTP(3)  = XC3                                      ! first cursor
      OUTP(4)  = XC4                                      ! last cursor 
      OUTP(5)  = SX                                       ! step in x
      OUTP(6)  = ALICO                                    ! line + cont
      OUTP(7)  = ACONT                                    ! continuum
      OUTP(8)  = ALINE                                    ! line
      OUTP(9)  = FLXFRC                                   ! flux fraction
      OUTP(10) = AEQW                                     ! equivalent width
      OUTP(11) = AEQWB                                  ! equivalent width blue
      OUTP(12) = AEQWR                                  ! equivalent width red
      CALL STKWRR('OUTPUTR',OUTP,1,12,KUN,ISTAT)
C
      RETURN
      END 







